"""
Tests transcribe_anything
"""

import os
import platform
import unittest

from transcribe_anything.api import fix_subtitles_path

HERE = os.path.abspath(os.path.dirname(__file__))

_IS_WINDOWS = platform.system() == "Windows"


class FixSubtitlesTester(unittest.TestCase):
    """Tester for transcribe anything."""

    @unittest.skipIf(not _IS_WINDOWS, "Test only works on Windows")
    def test_local_file(self) -> None:
        """Check that the command works on a local file."""
        path = r"C:\Users\Toshiba\Pictures\test vcp\shopi.mp4"
        fixed_path = fix_subtitles_path(path)
        expected_path = r"C\\:/\Users/\Toshiba/\Pictures/\test vcp/\shopi.mp4"
        self.assertEqual(fixed_path, expected_path)


if __name__ == "__main__":
    unittest.main()



"""
Runs whisper api.
"""

import subprocess
import sys
import time
from pathlib import Path
from typing import Optional

from iso_env import IsoEnv, IsoEnvArgs, PyProjectToml  # type: ignore

from transcribe_anything.util import has_nvidia_smi

# from isolated_environment import isolated_environment  # type: ignore


HERE = Path(__file__).parent
CUDA_AVAILABLE: Optional[bool] = None

# Set the versions
TENSOR_VERSION = "2.2.1"
CUDA_VERSION = "cu121"
EXTRA_INDEX_URL = f"https://download.pytorch.org/whl/{CUDA_VERSION}"

IS_MAC = sys.platform == "darwin"


def get_environment() -> IsoEnv:
    """Returns the environment."""
    venv_dir = HERE / "venv" / "whisper"
    needs_extra_index = not IS_MAC and has_nvidia_smi()
    content_lines: list[str] = []

    content_lines.append("[build-system]")
    content_lines.append('requires = ["setuptools", "wheel"]')
    content_lines.append('build-backend = "setuptools.build_meta"')
    content_lines.append("")

    content_lines.append("[project]")
    content_lines.append('name = "project"')
    content_lines.append('version = "0.1.0"')
    content_lines.append('requires-python = "==3.10.*"')
    content_lines.append("dependencies = [")
    content_lines.append('  "openai-whisper==20240930",')
    content_lines.append('  "numpy==1.26.4",')
    # f"torch=={TENSOR_VERSION}"
    if needs_extra_index:
        content_lines.append(f'  "torch=={TENSOR_VERSION}+{CUDA_VERSION}",')
    content_lines.append("]")

    needs_extra_index = not IS_MAC and has_nvidia_smi()
    if needs_extra_index:
        content_lines.append("[tool.uv.sources]")
        content_lines.append("torch = [")
        content_lines.append("  { index = 'pytorch-cu121' },")
        content_lines.append("]")
        content_lines.append("[[tool.uv.index]]")
        content_lines.append('name = "pytorch-cu121"')
        content_lines.append(f'url = "{EXTRA_INDEX_URL}"')
        content_lines.append("explicit = true")

    # if not IS_MAC and has_nvidia_smi():
    #     deps.append(
    #         f"torch=={TENSOR_VERSION}+{CUDA_VERSION} --extra-index-url {EXTRA_INDEX_URL}"
    #     )
    # else:
    #     deps.append(f"torch=={TENSOR_VERSION}")
    content = "\n".join(content_lines)
    pyproject_toml = PyProjectToml(content)
    args = IsoEnvArgs(venv_dir, build_info=pyproject_toml)
    env = IsoEnv(args)
    return env


def get_computing_device() -> str:
    """Get the computing device."""
    global CUDA_AVAILABLE  # pylint: disable=global-statement
    if CUDA_AVAILABLE is None:
        env = get_environment()
        py_file = HERE / "cuda_available.py"
        cp = env.run([py_file], shell=False, check=False)
        CUDA_AVAILABLE = cp.returncode == 0
    return "cuda" if CUDA_AVAILABLE else "cpu"


def run_whisper(  # pylint: disable=too-many-arguments
    input_wav: Path,
    device: str,
    model: str,
    output_dir: Path,
    task: str,
    language: str,
    other_args: list[str] | None = None,
) -> None:
    """Runs whisper."""
    env = get_environment()
    cmd_list = []
    # if sys.platform == "win32":
    #     # Set the text mode to UTF-8 on Windows.
    #     cmd_list.extend(["C:\Windows\System32\chcp.com", "65001", "&&"])
    cmd_list.append("whisper")
    cmd_list.append(str(input_wav))
    cmd_list.append("--device")
    cmd_list.append(device)
    if model:
        cmd_list.append("--model")
        cmd_list.append(model)
    cmd_list.append("--output_dir")
    cmd_list.append(str(output_dir))
    cmd_list.append("--task")
    cmd_list.append(task)
    if language:
        # cmd_list.append(f'--language "{language}"')
        cmd_list.append("--language")
        cmd_list.append(language)

    if other_args:
        cmd_list.extend(other_args)
    # Remove the empty strings.
    cmd_list = [str(x).strip() for x in cmd_list if str(x).strip()]
    # cmd = " ".join(cmd_list)
    cmd = subprocess.list2cmdline(cmd_list)
    sys.stderr.write(f"Running:\n  {cmd}\n")
    proc = env.open_proc(cmd_list, shell=False)
    while True:
        rtn = proc.poll()
        if rtn is None:
            time.sleep(0.25)
            continue
        if rtn != 0:
            msg = f"Failed to execute {cmd}\n "
            raise OSError(msg)
        break



"""
Fetches audio and handles transcoding it for it.
"""

import os
import shutil
import subprocess
import sys
import tempfile

from transcribe_anything.util import PROCESS_TIMEOUT
from transcribe_anything.ytldp_download import ytdlp_download


def _convert_to_wav(inpath: str, outpath: str, speech_normalization: bool = False) -> None:
    """Converts a file to wav."""
    # static_ffmpeg -y -i C:\Users\niteris\AppData\Local\Temp\tmp3xhzm1sn\out.webm -filter:a "speechnorm=e=12.5:r=0.00001:l=1" -acodec pcm_s16le -ar 44100 -ac 1 C:\Users\niteris\AppData\Local\Temp\tmpu32zsjov.wav

    tmpwav = tempfile.NamedTemporaryFile(suffix=".wav", delete=False)  # pylint: disable=consider-using-with
    tmpwav.close()
    tmpwavepath = tmpwav.name

    cmd_list = ["static_ffmpeg", "-y", "-i", str(inpath)]
    if speech_normalization:
        cmd_list += [
            "-filter:a",
            "speechnorm=e=12.5:r=0.00001:l=1",
        ]
    cmd_list += ["-acodec", "pcm_s16le", "-ar", "44100", "-ac", "1", str(tmpwavepath)]
    cmd = subprocess.list2cmdline(cmd_list)
    print(f"Running:\n  {cmd}")
    try:
        subprocess.run(
            cmd,
            shell=True,
            check=False,
            capture_output=True,
            timeout=PROCESS_TIMEOUT,
        )
    except subprocess.CalledProcessError as exc:
        print(f"Failed to run {cmd} with error {exc}")
        print(f"stdout: {exc.stdout}")
        print(f"stderr: {exc.stderr}")
        raise
    os.remove(outpath)
    # os.rename(tmpwavepath, outpath)
    # overwrite file at outpath
    shutil.copyfile(tmpwavepath, outpath)
    assert os.path.exists(outpath), f"The expected file {outpath} doesn't exist"


def fetch_audio(url_or_file: str, out_wav: str) -> None:
    """Fetches from the internet or from a local file and outputs a wav file."""
    assert out_wav.endswith(".wav")
    if url_or_file.startswith("http") or url_or_file.startswith("ftp"):
        with tempfile.TemporaryDirectory() as tmpdir:
            print(f"Using temporary directory {tmpdir}")
            downloaded_file = ytdlp_download(url_or_file, os.path.abspath(tmpdir))
            print("Downloaded file: ", downloaded_file)
            _convert_to_wav(downloaded_file, out_wav, speech_normalization=True)
        sys.stderr.write("Downloading complete.\n")
        assert os.path.exists(out_wav), f"The expected file {out_wav} doesn't exist"
    else:
        assert os.path.isfile(url_or_file)
        abspath = os.path.abspath(url_or_file)
        out_wav_abs = os.path.abspath(out_wav)
        with tempfile.TemporaryDirectory() as tmpdir:
            static_ffmpeg_path = shutil.which("static_ffmpeg")
            if static_ffmpeg_path is None:
                raise FileNotFoundError("No path for static_ffmpeg")
            cmd_list = [
                static_ffmpeg_path,
                "-y",
                "-i",
                str(abspath),
                "-acodec",
                "pcm_s16le",
                "-ar",
                "44100",
                "-ac",
                "1",
                "out.wav",
            ]
            cmd_str = subprocess.list2cmdline(cmd_list)
            sys.stderr.write(f"Running:\n  {cmd_str}\n")
            try:
                subprocess.run(
                    cmd_list,
                    cwd=tmpdir,
                    shell=False,
                    check=False,
                    capture_output=True,
                    timeout=PROCESS_TIMEOUT,
                )
                shutil.copyfile(os.path.join(tmpdir, "out.wav"), out_wav_abs)
            except subprocess.CalledProcessError as exc:
                print(f"Failed to run {cmd_str} with error {exc}")
                print(f"stdout: {exc.stdout.decode()}")
                print(f"stderr: {exc.stderr.decode()}")
                raise
        assert os.path.exists(out_wav), f"The expected file {out_wav} doesn't exist"


def unit_test() -> None:
    """Runs the program."""
    url = "https://www.youtube.com/watch?v=8Wg8f2g_GQY"
    fetch_audio(url, "out.wav")


if __name__ == "__main__":
    unit_test()



"""
Requirements for the insanely fast whisper.
"""

import sys
from pathlib import Path

from iso_env import IsoEnv, IsoEnvArgs, PyProjectToml  # type: ignore

from transcribe_anything.util import has_nvidia_smi

HERE = Path(__file__).parent

# Set the versions
TENSOR_VERSION = "2.1.2"
CUDA_VERSION = "cu121"
TENSOR_CUDA_VERSION = f"{TENSOR_VERSION}+{CUDA_VERSION}"
EXTRA_INDEX_URL = f"https://download.pytorch.org/whl/{CUDA_VERSION}"


def get_current_python_version() -> str:
    """Returns the current python version."""
    return f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"


# for insanely fast whisper, use:
#   pipx install insanely-fast-whisper --python python3.11


def get_environment() -> IsoEnv:
    """Returns the environment."""
    venv_dir = HERE / "venv" / "insanely_fast_whisper"
    deps = [
        "openai-whisper==20240930",
        "insanely-fast-whisper==0.0.15",
        "torchaudio==2.1.2",
        "datasets==2.17.1",
        "pytorch-lightning==2.1.4",
        "torchmetrics~=1.3.0",
        "srtranslator==0.2.6",
        "numpy==1.26.4",
    ]

    content_lines: list[str] = []

    content_lines.append("[build-system]")
    content_lines.append('requires = ["setuptools", "wheel"]')
    content_lines.append('build-backend = "setuptools.build_meta"')
    content_lines.append("")

    content_lines.append("[project]")
    content_lines.append('name = "project"')
    content_lines.append('version = "0.1.0"')
    content_lines.append('requires-python = "==3.10.*"')
    content_lines.append("dependencies = [")
    for dep in deps:
        content_lines.append(f'  "{dep}",')
    needs_extra_index = False
    if has_nvidia_smi():
        needs_extra_index = True
        content_lines.append(f'  "torch=={TENSOR_CUDA_VERSION}",')
    else:
        content_lines.append(f'  "torch=={TENSOR_VERSION}",')
    if sys.platform != "darwin":
        # Add the windows specific dependencies.
        content_lines.append('  "intel-openmp==2024.0.3",')
    content_lines.append("]")
    content_lines.append("")

    # if has_nvidia_smi():
    #     deps.append(f"torch=={TENSOR_CUDA_VERSION} --extra-index-url {EXTRA_INDEX_URL}")
    # else:
    #     deps.append(f"torch=={TENSOR_VERSION}")
    # if sys.platform != "darwin":
    #     # Add the windows specific dependencies.
    #     deps.append("intel-openmp==2024.0.3")

    if needs_extra_index:
        # [tool.uv.sources]
        #   torch = [
        #   { index = "pytorch-cu121", marker = "platform_system == 'Windows'" },
        # ]
        content_lines.append("[tool.uv.sources]")
        content_lines.append("torch = [")
        content_lines.append("  { index = 'pytorch-cu121' },")
        content_lines.append("]")

        # [[tool.uv.index]]
        # name = "pytorch-cu121"
        # url = "https://download.pytorch.org/whl/cu121"
        # explicit = true

        content_lines.append("[[tool.uv.index]]")
        content_lines.append('name = "pytorch-cu121"')
        content_lines.append(f'url = "{EXTRA_INDEX_URL}"')
        content_lines.append("explicit = true")

        # deps.append(f"--extra-index-url {EXTRA_INDEX_URL}")

    content = "\n".join(content_lines)
    build_info = PyProjectToml(content)
    args = IsoEnvArgs(venv_path=venv_dir, build_info=build_info)
    env = IsoEnv(args)
    return env



# """
# Installs whisper in an isolated environment.

# This is a just a test. It is not used in code.
# """

# import os
# from pathlib import Path

# TENSOR_VERSION = "2.1.2"
# CUDA_VERSION = "cu121"
# TENSOR_CUDA_VERSION = f"{TENSOR_VERSION}+{CUDA_VERSION}"
# EXTRA_INDEX_URL = f"https://download.pytorch.org/whl/{CUDA_VERSION}"

# HERE = Path(os.path.abspath(os.path.dirname(__file__)))


# def unit_test() -> None:
#     """Unit test."""
#     from tempfile import TemporaryDirectory  # pylint: disable=import-outside-toplevel

#     with TemporaryDirectory() as tmpdir:
#         print(f"Using temporary directory {tmpdir}")
#         iso_env = IsolatedEnvironment(HERE / "transcribe_anything_env")
#         iso_env.pip_install(
#             package=f"torch=={TENSOR_VERSION}",
#             build_options=f"--extra-index-url {EXTRA_INDEX_URL}",
#             full_isolation=True,
#         )
#         iso_env.pip_install(
#             "torchaudio==2.1.2", build_options=None, full_isolation=True
#         )
#         iso_env.pip_install(
#             package="openai-whisper", build_options=None, full_isolation=True
#         )
#         iso_env.pip_install(
#             package="numpy==1.26.4", build_options=None, full_isolation=True
#         )
#         iso_env.run(["whisper", "--help"])


# if __name__ == "__main__":
#     unit_test()



# pylint: skip-file

"""
DEPRECATED: This script should no longer be necessary do to isolated environments,
but I'm keeping it around for now just in case.
Installs cuda + transcibe-anything
"""

# flake8: noqa E501

import argparse
import os
import shutil
import subprocess
import sys

HERE = os.path.dirname(os.path.abspath(__file__))
API_PY = os.path.join(HERE, "api.py")

# Set the versions
TENSOR_VERSION = "2.1.2"
CUDA_VERSION = "cu121"
TENSOR_CUDA_VERSION = f"{TENSOR_VERSION}+{CUDA_VERSION}"
EXTRA_INDEX_URL = f"https://download.pytorch.org/whl/{CUDA_VERSION}"


argparser = argparse.ArgumentParser()
argparser.add_argument("--display-cuda-version", default=CUDA_VERSION, help="displays cuda version")
argparser.add_argument(
    "--force", action="store_true", help="Force install and ignore missing nvida drivers"
)
args = argparser.parse_args()

HAS_NVIDIA: bool = shutil.which("nvidia-smi") is not None

if not HAS_NVIDIA and not args.force:
    print("No nvidia-smi detected, skipping cuda install")
    sys.exit(1)

# Get the stdout from pip list
pip_list_stdout = subprocess.run(
    ["pip", "list", "--format", "json"], check=True, capture_output=True, universal_newlines=True
).stdout

# Delete the torch package if it doesn't have the cuda version
if args.force or True:  # always do this path now.  # pylint: disable=condition-evals-to-constant
    uninstall = ["torch", "torchtext", "torchdata", "torchaudio", "torchvision", "torch-directml"]
    for package in uninstall:
        subprocess.run(["pip", "uninstall", "-y", package], check=True)
    subprocess.run(["pip", "cache", "purge"], check=True)
else:
    if TENSOR_CUDA_VERSION not in pip_list_stdout:
        print(f"The substring '${TENSOR_CUDA_VERSION}' does not exist in the string.")
        subprocess.run(["pip", "uninstall", "-y", "torch"], check=True)
        print("Purging pip cache to remove any torch packages that are cpu only")
        subprocess.run(["pip", "cache", "purge"], check=True)
    else:
        print(f"Tensorflow {TENSOR_CUDA_VERSION} is currently installed")

# Install torch with cuda
print("Installing torch+cuda")
subprocess.run(
    [
        "pip",
        "install",
        f"torch=={TENSOR_VERSION}",
        "--extra-index-url",
        EXTRA_INDEX_URL,
    ],
    check=True,
)


def use_local_install() -> bool:
    """Prompts the user for the install path, if we detect that this is in a github action"""

    if os.path.exists(API_PY):
        use_local = input("Use local install? [y/n] ").lower() == "y"
        if use_local:
            return True
    return False


USE_LOCAL_INSTALL = use_local_install()
if USE_LOCAL_INSTALL:
    install_cmd = ["pip", "install", "-e", "."]
else:
    install_cmd = ["pip", "install", "transcribe-anything"]

# Install transcribe_anything
print("Install transcribe_anything:\n", " ".join(install_cmd))
subprocess.run(install_cmd, check=True)

print("\ntranscribe audio is installed, run it with\n  transcribe_audio <URL OR FILE>")



"""
Utilities for srt translation, including srt wrap.
"""

import subprocess
import warnings
from pathlib import Path

from iso_env import IsoEnv, IsoEnvArgs, Requirements  # type: ignore

# from isolated_environment import isolated_environment  # type: ignore


HERE = Path(__file__).parent
WRAP_SRT_PY = HERE / "srt_wrap.py"


def get_environment() -> IsoEnv:
    """Returns the environment."""
    venv_path = HERE / "venv" / "srttranslator"
    reqs_text = "\n".join(["srtranslator==0.3.9", "requests==2.28.1", "urllib3==1.26.13"])
    reqs = Requirements(
        reqs_text,
        python_version="==3.11.*",
    )
    args = IsoEnvArgs(
        venv_path=venv_path,
        build_info=reqs,
    )
    env = IsoEnv(args)
    return env


def srt_wrap_to_string(srt_file: Path) -> str:
    """Wrap lines in a srt file."""
    env = get_environment()
    cmd_list = [
        str(WRAP_SRT_PY),
        str(srt_file),
    ]
    try:
        cp: subprocess.CompletedProcess = env.run(
            cmd_list,
            check=True,
            capture_output=True,
            text=True,
            shell=False,
        )
    except subprocess.CalledProcessError as exc:
        stderr = exc.stderr
        error_msg = f"Failed to run srt_wrap: {exc}"
        error_msg += f"\n{stderr}"
        warnings.warn(error_msg)
        raise
    out = cp.stdout
    return out


def srt_wrap(srt_file: Path) -> None:
    """Wrap lines in a srt file."""
    try:
        assert WRAP_SRT_PY.exists()
        out = srt_wrap_to_string(srt_file)
        srt_file.write_text(out, encoding="utf-8")
    except subprocess.CalledProcessError as exc:
        warnings.warn(f"Failed to run srt_wrap: {exc}")



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access

import os
import shutil
import subprocess
import unittest

HERE = os.path.abspath(os.path.dirname(__file__))
LOCALFILE_DIR = os.path.join(HERE, "localfile")
TESTS_DATA_DIR = os.path.join(LOCALFILE_DIR, "text_video", "en")


class TranscribeAnythingTester(unittest.TestCase):
    """Tester for transcribe anything."""

    def test_local_file(self) -> None:
        """Check that the command works on a local file."""
        shutil.rmtree(TESTS_DATA_DIR, ignore_errors=True)
        cmd_list: list[str] = [
            "transcribe_anything",
            "video.mp4",
            "--language",
            "en",
            "--model",
            "tiny",
        ]
        try:
            cmd_str = subprocess.list2cmdline(cmd_list)
            print(f"Running in {LOCALFILE_DIR}: {cmd_str}")
            subprocess.run(cmd_list, cwd=LOCALFILE_DIR, check=True)
        except subprocess.CalledProcessError as e:  # pylint: disable=R0801
            print(e.output)
            raise e


if __name__ == "__main__":
    unittest.main()
